"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createChart = void 0;

var _react = _interopRequireWildcard(require("react"));

var _useDeepCompare = require("use-deep-compare");

var _components = require("./components");

var _jsxRuntime = require("react/jsx-runtime");

const _excluded = ["data", "onElementClick"];

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

const createChart = (Chart, {
  transformData,
  transformConfig
}) => /*#__PURE__*/(0, _react.memo)(props => {
  const {
    data,
    onElementClick
  } = props,
        config = _objectWithoutProperties(props, _excluded);

  const chartRef = (0, _react.useRef)(null);
  const chartContainerRef = (0, _react.useRef)(null);
  const propsRef = (0, _react.useRef)(props);
  propsRef.current = props;
  const handleElementClick = (0, _react.useCallback)(event => {
    onElementClick(event.data);
  }, [onElementClick]); // 图表实例初始化

  (0, _react.useEffect)(() => {
    if (!chartContainerRef.current) {
      return () => null;
    }

    const chartConfig = typeof transformConfig === 'function' ? transformConfig(propsRef.current) : config;
    const chartData = typeof transformData === 'function' ? transformData(propsRef.current) : data;
    chartRef.current = new Chart(chartContainerRef.current, _objectSpread(_objectSpread({}, chartConfig), {}, {
      data: chartData
    }));

    if (propsRef.current.onElementClick) {
      var _chartRef$current;

      (_chartRef$current = chartRef.current) === null || _chartRef$current === void 0 ? void 0 : _chartRef$current.on('element:click', handleElementClick);
    }

    return () => {
      var _chartRef$current2, _chartRef$current3;

      (_chartRef$current2 = chartRef.current) === null || _chartRef$current2 === void 0 ? void 0 : _chartRef$current2.off('element:click', handleElementClick);
      (_chartRef$current3 = chartRef.current) === null || _chartRef$current3 === void 0 ? void 0 : _chartRef$current3.destroy();
    };
  }, [chartContainerRef.current, handleElementClick]); // 更新图表数据、配置

  (0, _useDeepCompare.useDeepCompareEffect)(() => {
    // TODO 优化点：第一次渲染数据为空不渲染，之后数据为空需渲染为空
    if (!chartRef.current || !data) {
      return;
    }

    try {
      const chartData = typeof transformData === 'function' ? transformData(propsRef.current) : data;
      const chartConfig = typeof transformConfig === 'function' ? transformConfig(propsRef.current) : config;
      window.requestIdleCallback(() => {
        var _chartRef$current4;

        (_chartRef$current4 = chartRef.current) === null || _chartRef$current4 === void 0 ? void 0 : _chartRef$current4.update(_objectSpread(_objectSpread({}, chartConfig), {}, {
          data: chartData
        }));
      });
    } catch (_unused) {
      console.error('chart update found error');
    }
  }, [data, config]);
  const Title = /*#__PURE__*/(0, _jsxRuntime.jsxs)("div", {
    className: "title",
    style: {
      marginBottom: '20px'
    },
    children: [/*#__PURE__*/(0, _jsxRuntime.jsx)("h1", {
      style: {
        marginBottom: '-10px'
      },
      children: config.title
    }), /*#__PURE__*/(0, _jsxRuntime.jsx)("h2", {
      style: {
        color: 'rgb(78, 89, 105)'
      },
      children: config.subTitle
    })]
  });
  return /*#__PURE__*/(0, _jsxRuntime.jsxs)("div", {
    className: "relative",
    children: [Title, /*#__PURE__*/(0, _jsxRuntime.jsx)("div", {
      ref: chartContainerRef
    }), !(data !== null && data !== void 0 && data.length) ? /*#__PURE__*/(0, _jsxRuntime.jsx)(_components.ChartEmpty, {
      children: "\u6682\u65E0\u6570\u636E"
    }) : null]
  });
});

exports.createChart = createChart;