import { createBlock } from '@modern-js-block/runtime';
import { Funnel, FunnelOptions } from '@antv/g2plot';
import { createChart, SeqChartOptions } from '@/base';

export type FunnelChartProps = SeqChartOptions<FunnelOptions>;

const rateFormatter = (v: number) => `${(v * 100).toFixed(2)}%`;

const conversionTag = {
  offsetX: 10,
  offsetY: 0,
  formatter: (datum: any) => {
    const [pre, curr] = datum?.compareNumber || [1, 1];
    return `转化率${rateFormatter(curr / pre)}`;
  },
};

const label = {
  formatter: (_text: any, item: any) =>
    `${item._origin.stage} ${item._origin.number}`,
};

const tooltip = {
  customItems: (items: any) =>
    items.map((d: any) => ({ ...d, value: d.data.number })),
};

const FunnelChart = createChart<FunnelChartProps>(Funnel, {
  transformData: (options: any) => {
    const { data, xField, yField } = options;
    // 把数据统一转换为{ stage, number }
    const newData =
      data?.map((it: any, idx: number) => ({
        ...it,
        stage: data[idx][xField],
        number: Number(data[idx][yField]),
        compareNumber: [
          data?.[idx - 1]?.[yField] || undefined,
          data[idx][yField],
        ],
      })) || [];

    // const total = newData?.reduce(
    //   (acc: number, it: Record<string, any>) => it.number + acc,
    //   0,
    // );
    if (newData && xField && yField) {
      newData?.forEach((it: any) => {
        // it[yField] = Number(it[yField]);
        // it.totalNumber = total;
        it._number = 1;
      });
    }
    return newData || [];
  },
  transformConfig: options => {
    const newOptions = {
      ...options,
      yField: '_number',
      label,
      tooltip,
      conversionTag,
    };
    return newOptions;
  },
});

export default createBlock<FunnelChartProps>({
  xField: 'stage',
  yField: 'number',
  legend: false,
  data: [],
  title: '漏斗图',
  subTitle: '',
  dynamicHeight: true,
})(FunnelChart);
