import { createEditor, PluginParams } from '@modern-js-block/runtime';
import HeatmapChart from '@/HeatmapChart';

createEditor(HeatmapChart, (params: PluginParams) => {
  const { ControlType, addPropertyControls } = params;
  addPropertyControls({
    title: {
      type: ControlType.String,
      title: '主标题',
    },
    subTitle: {
      type: ControlType.String,
      title: '子标题',
    },
    width: {
      type: ControlType.Number,
      title: '宽度',
    },
    height: {
      type: ControlType.Number,
      title: '高度',
    },
    data: {
      type: ControlType.Json,
      title: '图表数据',
    },
    isSorted: {
      type: ControlType.Boolean,
      title: '是否默认以x轴排序',
      defaultValue: false,
      tips: '大量数据情况可能会减慢渲染速度',
    },
    xField: {
      type: ControlType.String,
      title: 'x轴',
    },
    yField: {
      type: ControlType.String,
      title: 'y轴',
    },
    colorField: {
      type: ControlType.String,
      title: '颜色字段',
    },
    meta: {
      type: ControlType.Json,
      title: '自定义meta信息',
      tips: '可参考https://antv-g2plot.gitee.io/zh/docs/api/options/meta',
      defaultValue: {
        'Month of Year': {
          type: 'cat',
        },
      },
    },
    color: {
      type: ControlType.Array,
      title: '颜色区间',
      tips: '靠前的颜色对应数值小的数据',
      propertyControl: {
        type: ControlType.Color,
        title: '颜色',
        defaultValue: '#000000',
      },
    },
    onElementClick: {
      type: ControlType.Action,
      title: '点击元素单元',
      params: {
        value: {
          type: ControlType.Object,
          title: '热力图元素数据',
        },
      },
    },
  });
});
